@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2018 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2020 -- leonerd@leonerd.org.uk
 
 package Tickit::Term;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.67_002';
+our $VERSION = '0.70_001';
 
 use Carp;
 
@@ -48,8 +48,21 @@ This object is not normally constructed directly by the containing
 application; instead it is used indirectly by other parts of the C<Tickit>
 distribution.
 
+Note that a given program may contain multiple objects in this class that all
+refer to the same underlying C<TickitTerm> instance from the C library. This
+is especially true of the first argument provided to event binding callbacks.
+This class overloads numify and stringify operations, so that instances may be
+compared using the C<==> or C<eq> operators, or used as keys in hashes, and
+they will act as expected. Do not rely on plain C<refaddr> comparison however
+as you may get incorrect results.
+
 =cut
 
+use overload
+   '0+' => "_xs_addr",
+   '""' => sub { sprintf "Tickit::Term=XS(tt=0x%x)", $_[0]->_xs_addr },
+   fallback => 1;
+
 =head1 CONSTRUCTOR
 
 =cut
@@ -100,17 +113,9 @@ sub new
    my $class = shift;
    my %params = @_;
 
-   my $self = $class->_new( $ENV{TERM} ) or croak "Cannot construct Tickit::Term - $!";
-
-   $self->set_input_handle ( $params{input_handle}  ) if $params{input_handle};
-   $self->set_output_handle( $params{output_handle} ) if $params{output_handle};
-
-   my $writer = $params{writer};
-   $self->set_output_func( sub { $writer->write( @_ ) } ) if $writer;
-
-   $self->set_utf8( $params{UTF8} ) if defined $params{UTF8};
-
-   return $self;
+   return $class->_new(
+      $ENV{TERM}, @params{qw( input_handle output_handle writer UTF8 )}
+   ) || croak "Cannot construct Tickit::Term - $!";
 }
 
 =head2 open_stdio
@@ -248,6 +253,10 @@ more of the following constants:
 
 Inserts this event handler first in the chain, before any existing ones.
 
+=item TICKIT_BIND_ONESHOT
+
+Remove the event handler after it has been invoked the first time.
+
 =back
 
 =head2 unbind_event_id
